<!DOCTYPE html>
<html lang="en">

<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <meta http-equiv="X-UA-Compatible" content="ie=edge">
    <link rel="stylesheet" href="{{ url_for('static', filename='css/bootstrap.min.css') }}">
    <link rel="stylesheet" href="{{ url_for('static', filename='css/index.css') }}">
    <link rel="shortcut icon" href="{{ url_for('static', filename='images/favicon.ico') }}">
    <title>Oops!</title>
</head>

<body>
    <div class="container">
        <div class="d-flex justify-content-center mt-5">
            <div class="text-center">
                <img class="img-responsive" src="{{ url_for('static', filename='images/graphenex_logo_white.png') }}"
                    alt="logo" height="200" width="200" id="banner">
                <h3>grapheneX</h3><hr class="fancy-hr">
                <h1>404</h1>
                <h4>You seem lost.</h4> 
            </div>
        </div>
    </div>

    <script src="{{ url_for('static', filename='js/bootstrap.min.js') }}"></script>
</body>

</html>